/*
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include <dcm_image_codec_debug.h>
#include <dcm_image_codec.h>
#include <mm_util_magick.h>

#define OPT_IMAGE_WIDTH		1280
#define OPT_IMAGE_HEIGHT	720

typedef enum {
	NOT_AVAILABLE = 0,
	NORMAL = 1,
	HFLIP = 2,
	ROT_180 = 3,
	VFLIP = 4,
	TRANSPOSE = 5,
	ROT_90 = 6,
	TRANSVERSE = 7,
	ROT_270 = 8
} dcm_exif_orientation_e;

static bool __dcm_get_optimized_wh(unsigned int src_width, unsigned int src_height, unsigned int *calc_width, unsigned int *calc_height)
{
	*calc_width = 0;
	*calc_height = 0;

	if (src_width <= OPT_IMAGE_WIDTH && src_height <= OPT_IMAGE_HEIGHT) {
		dcm_debug("Original image is smaller than target image");
		*calc_width = src_width;
		*calc_height = src_height;
		return FALSE;
	}

	if (src_width > src_height) {
		*calc_width = OPT_IMAGE_WIDTH;
		*calc_height = (int)(src_height * (((double) OPT_IMAGE_WIDTH) / ((double) src_width)));
	} else {
		*calc_width = (int)(src_width * (((double) OPT_IMAGE_HEIGHT) / ((double) src_height)));
		*calc_height = OPT_IMAGE_HEIGHT;
	}

	return TRUE;
}

int dcm_decode_image(const char *file_path, const int orientation, const gboolean resize, unsigned char **image_buffer, size_t *size,
	unsigned int *buff_width, unsigned int *buff_height)
{
	int ret = MS_MEDIA_ERR_NONE;
	unsigned int width = 0;
	unsigned int height = 0;
	unsigned int resized_width = 0;
	unsigned int resized_height = 0;
	mm_util_img_codec_type type = IMG_CODEC_UNKNOWN_TYPE;
	mm_util_magick_rotate_type angle = MM_UTIL_ROTATE_NUM;
	mm_util_image_h resize_dst_handle = NULL;
	mm_util_image_h dst_handle = NULL;
	mm_util_color_format_e format = MM_UTIL_COLOR_NUM;

	dcm_debug_fenter();

	DCM_CHECK_VAL(file_path, MS_MEDIA_ERR_INVALID_PARAMETER);

	if (orientation == NOT_AVAILABLE || orientation == NORMAL)
		angle = MM_UTIL_ROTATE_0;
	else if (orientation == ROT_90)
		angle = MM_UTIL_ROTATE_90;
	else if (orientation == ROT_180)
		angle = MM_UTIL_ROTATE_180;
	else if (orientation == ROT_270)
		angle = MM_UTIL_ROTATE_270;
	else if (orientation == HFLIP)
		angle = MM_UTIL_ROTATE_FLIP_HORZ;
	else if (orientation == VFLIP)
		angle = MM_UTIL_ROTATE_FLIP_VERT;
	else {
		dcm_error("Not supported angle [%d]", orientation);
		return MS_MEDIA_ERR_INTERNAL;
	}

	if (resize) {
		/* Get the optimized width/height to enhance performance for big size image */
		ret = mm_util_extract_image_info(file_path, &type, &width, &height);
		if (ret != MM_UTIL_ERROR_NONE)
			dcm_error("Failed to get image info!err: %d", ret);

		if (__dcm_get_optimized_wh(width, height, &resized_width, &resized_height)) {
			/* Resize the big size image to enhance performance for big size image */
			ret = mm_util_resize_P_B(file_path, resized_width, resized_height, MM_UTIL_COLOR_RGBA, &resize_dst_handle);
			if (ret != MM_UTIL_ERROR_NONE) {
				dcm_error("Failed to resize image!err: %d", ret);
				ret = MS_MEDIA_ERR_INTERNAL;
				goto ERROR;
			}

			ret = mm_util_rotate_B_B(resize_dst_handle, angle, &dst_handle);
			if (ret != MM_UTIL_ERROR_NONE) {
				dcm_error("Failed to rotate image!err: %d", ret);
				ret = MS_MEDIA_ERR_INTERNAL;
				goto ERROR;
			}
		} else {
			ret = mm_util_rotate_P_B(file_path, angle, MM_UTIL_COLOR_RGBA, &dst_handle);
			if (ret != MM_UTIL_ERROR_NONE) {
				dcm_error("Failed to rotate image buffer! err: %d", ret);
				ret = MS_MEDIA_ERR_INTERNAL;
				goto ERROR;
			}
		}
	} else {
		ret = mm_util_rotate_P_B(file_path, angle, MM_UTIL_COLOR_RGBA, &dst_handle);
		if (ret != MM_UTIL_ERROR_NONE) {
			dcm_error("Failed to rotate image buffer! err: %d", ret);
			ret = MS_MEDIA_ERR_INTERNAL;
			goto ERROR;
		}
	}

	mm_image_get_image(dst_handle, buff_width, buff_height, &format, image_buffer, size);

ERROR:

	if (resize_dst_handle)
		mm_image_destroy_image(resize_dst_handle);
	if (dst_handle)
		mm_image_destroy_image(dst_handle);

	dcm_debug_fleave();

	return ret;
}
