/*
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <dcm-face.h>
#include <dcm_svc_db.h>
#include <dcm_svc_debug.h>
#include <dcm_svc_detect_face.h>

static dcm_face_h dcm_face_handle = NULL;

static double __calculate_scale_factor(unsigned int original_width, unsigned int original_height, unsigned int buffer_width, unsigned int buffer_height)
{
	double scale_factor = 0.0;

	if (original_width >= original_height) {
		if (buffer_width >= buffer_height)
			scale_factor = ((double) (original_width)) / ((double) (buffer_width));
		else
			scale_factor = ((double) (original_width)) / ((double) (buffer_height));
	} else {
		if (buffer_height >= buffer_width)
			scale_factor = ((double) (original_height)) / ((double) (buffer_height));
		else
			scale_factor = ((double) (original_height)) / ((double) (buffer_width));
	}

	dcm_debug("scale_factor: [%lf]", scale_factor);

	return scale_factor;
}

int dcm_face_detect_initialize()
{
	int ret = MS_MEDIA_ERR_NONE;

	dcm_debug_fenter();

	ret = dcm_face_create(&dcm_face_handle);

	dcm_debug_fleave();

	return ret;
}

int dcm_face_detect_finalize()
{
	int ret = MS_MEDIA_ERR_NONE;

	dcm_debug_fenter();

	DCM_CHECK_VAL(dcm_face_handle, MS_MEDIA_ERR_INVALID_PARAMETER);

	ret = dcm_face_destroy(dcm_face_handle);

	dcm_face_handle = NULL;

	dcm_debug_fleave();

	return ret;
}

int dcm_face_detect_process(dcm_svc_item_s *scan_item, dcm_face_scan_status_e scan_status, dcm_image_info_s *image_info)
{
	int face_area = 0;
	int i = 0;
	double scale_factor = 0.0;
	int err = MS_MEDIA_ERR_NONE;
	int ret = MS_MEDIA_ERR_NONE;
	face_info_s *face_info = NULL;
	unsigned int face_rect_x = 0;
	unsigned int face_rect_y = 0;
	unsigned int face_rect_w = 0;
	unsigned int face_rect_h = 0;
	int face_orientation = 0;

	dcm_debug_fenter();

	DCM_CHECK_VAL(dcm_face_handle, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(scan_item, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(scan_item->media_uuid, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(image_info, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(image_info->pixel, MS_MEDIA_ERR_INVALID_PARAMETER);

	dcm_debug("colorspce: [%d], w: [%d], h: [%d]", image_info->decode_type, image_info->buffer_width, image_info->buffer_height);

	ret = dcm_face_set_image_info(dcm_face_handle, (face_image_colorspace_e)image_info->decode_type, image_info->pixel, image_info->buffer_width, image_info->buffer_height, image_info->size);
	if (ret != MS_MEDIA_ERR_NONE) {
		dcm_error("Failed to dcm_face_set_image_info! err: %d", err);
		goto DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED;
	}

	face_info = (face_info_s *)g_malloc0(sizeof(face_info_s));
	if (face_info == NULL) {
		dcm_error("Failed to allocate face info");
		ret = MS_MEDIA_ERR_OUT_OF_MEMORY;
		goto DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED;
	}

	ret = dcm_face_get_face_info(dcm_face_handle, face_info);
	if (ret != MS_MEDIA_ERR_NONE) {
		dcm_error("Failed to get face info! err: %d", err);
		goto DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED;
	}

	dcm_warn("detected face count: %d", face_info->count);
	if (face_info->count <= 0) {
		goto DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED;
	}

	/* Compute scale factor between decode size and original size */
	scale_factor = __calculate_scale_factor(image_info->original_width, image_info->original_height, image_info->buffer_width, image_info->buffer_height);

	/* Insert every face rectangle into database */
	for (i = 0; i < face_info->count; i++) {
		if (scale_factor > 1.0) {
			face_rect_x = (int) (face_info->rects[i].x * scale_factor);
			face_rect_y = (int) (face_info->rects[i].y * scale_factor);
			face_rect_w = (int) (face_info->rects[i].w * scale_factor);
			face_rect_h = (int) (face_info->rects[i].h * scale_factor);
		} else {
			face_rect_x = face_info->rects[i].x;
			face_rect_y = face_info->rects[i].y;
			face_rect_w = face_info->rects[i].w;
			face_rect_h = face_info->rects[i].h;
		}
		face_orientation = face_info->rects[i].orientation;

		face_area += face_rect_w * face_rect_h;

		dcm_debug("[#%d] face rect: X [%d] Y [%d] W [%d] H [%d] Orientation [%d]", i, face_rect_x, face_rect_y, face_rect_w, face_rect_h, face_orientation);

		/* Insert face rectangle into database */
		ret = dcm_svc_db_insert_face_to_db(scan_item->media_uuid, face_rect_x, face_rect_y, face_rect_w, face_rect_h, face_orientation);
		if (ret != MS_MEDIA_ERR_NONE) {
			dcm_error("Failed to insert face item into db! ret: %d", ret);
			goto DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED;
		}

	}

DCM_SVC_FACE_RECOGNIZE_BUFFER_FAILED:

	err = dcm_svc_db_insert_face_to_face_scan_list(scan_item);
	if (err != MS_MEDIA_ERR_NONE)
		dcm_error("Failed to insert face item into face_scan_list! err: %d", err);

	dcm_face_destroy_face_info(face_info);

	dcm_debug_fleave();

	return ret;
}
