/*
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <dcm_svc_ipc.h>
#include <dcm_svc_internal.h>
#include <dcm_svc_db.h>

#include <dcm_svc_debug.h>
#include <dcm_svc_detect_face.h>

static int __dcm_scan_process(dcm_svc_item_s *scan_item)
{
	dcm_face_scan_status_e scan_status = FACE_SCAN_STATUS_NONE;
	int ret = MS_MEDIA_ERR_NONE;

	dcm_debug_fenter();

	DCM_CHECK_VAL(scan_item, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(scan_item->file_path, MS_MEDIA_ERR_INVALID_PARAMETER);

	dcm_image_info_s image_info = {0, };
	memset(&image_info, 0, sizeof(dcm_image_info_s));
	image_info.decode_type = DCM_IMAGE_FORMAT_I420;

	if (g_file_test(scan_item->file_path, G_FILE_TEST_IS_REGULAR)) {
		ret = dcm_svc_db_check_scanned_by_media_uuid(scan_item->media_uuid, scan_item->modified_time, &scan_status);
		dcm_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Failed to get scan item info from db(%d)", ret);

		if (scan_status == FACE_SCAN_NO_NEEDED) {
			dcm_warn("This media is scanned already! Skip...");
			return MS_MEDIA_ERR_NONE;
		} else {
			dcm_debug("Need to Scan");
		}

		dcm_sec_debug("File path[%s] media id[%s]", scan_item->file_path, scan_item->media_uuid);

		image_info.original_width = scan_item->image_width;
		image_info.original_height = scan_item->image_height;
		image_info.orientation = scan_item->image_orientation;

		if (strcmp(scan_item->mime_type, MIME_TYPE_JPEG) == 0) {
			image_info.decode_type = DCM_IMAGE_FORMAT_RGBA;
		} else if ((strcmp(scan_item->mime_type, MIME_TYPE_PNG) == 0) ||
				(strcmp(scan_item->mime_type, MIME_TYPE_BMP) == 0)) {
			image_info.decode_type = DCM_IMAGE_FORMAT_RGBA;
		} else {
			dcm_error("Failed not supported type(%s)", scan_item->mime_type);
			return MS_MEDIA_ERR_INVALID_PARAMETER;
		}

		ret = dcm_decode_image((const char *) (scan_item->file_path), image_info.orientation, FALSE, &(image_info.pixel), &(image_info.size), &(image_info.buffer_width), &(image_info.buffer_height));
		dcm_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Failed dcm_decode_image(%d)", ret);

		dcm_info("Image info W[%d] H[%d] Orient[%d] buf_w[%d] buf_h[%d]",
			image_info.original_width, image_info.original_height, image_info.orientation, image_info.buffer_width, image_info.buffer_height);

		/* If status is refresh, remove all faces and re-insert */
		if (scan_status == FACE_SCAN_REFRESH_NEEDED) {
			ret = dcm_svc_db_delete_face_from_db(scan_item->media_uuid);
			if (ret != MS_MEDIA_ERR_NONE)
				dcm_error("Failed to dcm_svc_db_delete_face_from_db err: %d", ret);
		}

		/* Process face scan */
		ret = dcm_face_detect_process(scan_item, scan_status, &image_info);
		if (ret != MS_MEDIA_ERR_NONE)
			dcm_error("Failed to process face detection! err: %d", ret);

		/* Free image buffer */
		DCM_SAFE_FREE(image_info.pixel);
	} else {
		dcm_warn("The file does not exist");
		ret = MS_MEDIA_ERR_INVALID_PARAMETER;
	}

	dcm_debug_fleave();

	return ret;
}

int dcm_scan_single(const char *file_path, uid_t uid, int *face_count)
{
	int ret = MS_MEDIA_ERR_NONE;
	dcm_svc_item_s *scan_item = NULL;

	dcm_debug_fenter();

	DCM_CHECK_VAL(file_path, MS_MEDIA_ERR_INVALID_PARAMETER);
	DCM_CHECK_VAL(face_count, MS_MEDIA_ERR_INVALID_PARAMETER);

	ret = dcm_svc_db_connect(uid);
	dcm_retvm_if(ret != MS_MEDIA_ERR_NONE, ret, "Fail dcm_svc_db_connect");

	ret = dcm_svc_db_get_scan_image_info_by_path(file_path, &scan_item);
	if (ret != MS_MEDIA_ERR_NONE || scan_item == NULL) {
		dcm_error("Failed to dcm_svc_db_get_scan_image_info_by_path err: %d", ret);
		dcm_svc_db_disconnect();
		return ret;
	}

	ret = __dcm_scan_process(scan_item);
	if (ret != MS_MEDIA_ERR_NONE)
		dcm_error("Failed to process scan job! err: %d", ret);

	dcm_svc_db_get_face_cnt_from_db(scan_item->media_uuid, face_count);

	dcm_svc_db_disconnect();

	DCM_SAFE_FREE(scan_item->media_uuid);
	DCM_SAFE_FREE(scan_item->file_path);
	DCM_SAFE_FREE(scan_item);

	dcm_debug_fleave();

	return ret;
}

