/*
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <dcm_svc_ipc.h>
#include <dcm_svc_debug.h>
#include <tzplatform_config.h>

static char DCM_IPC_PATH[][100] = {
			{"media-server/media_ipc_dcmdaemon.socket"},
			{"media-server/media_ipc_dcmcomm.socket"},
};
#define DCM_MAX_RETRY 100

int dcm_ipc_receive_message(int client_sock, dcmMsg *recv_msg)
{
	if (read(client_sock, recv_msg, sizeof(dcmMsg)) != sizeof(dcmMsg)) {
		dcm_stderror("recv failed");
		return MS_MEDIA_ERR_IPC;
	}

	return MS_MEDIA_ERR_NONE;
}

int dcm_ipc_accept_socket(int serv_sock, int *client_sock)
{
	DCM_CHECK_VAL(client_sock, MS_MEDIA_ERR_INVALID_PARAMETER);
	int sockfd = -1;
	struct sockaddr_un client_addr;
	socklen_t client_addr_len = sizeof(client_addr);

	if ((sockfd = accept(serv_sock, (struct sockaddr*)&client_addr, &client_addr_len)) < 0) {
		dcm_stderror("accept failed");
		*client_sock = -1;
		return MS_MEDIA_ERR_IPC;
	}

	*client_sock = sockfd;

	return MS_MEDIA_ERR_NONE;
}

int dcm_ipc_create_socket(int *socket_fd, dcm_ipc_port_e port)
{
	DCM_CHECK_VAL(socket_fd, MS_MEDIA_ERR_INVALID_PARAMETER);
	int sock = -1;
	struct sockaddr_un serv_addr;
	int i = 0;

	/* Create a new TCP socket */
	if ((sock = socket(PF_FILE, SOCK_STREAM, 0)) < 0) {
		dcm_stderror("socket failed");
		return MS_MEDIA_ERR_IPC;
	}

	/* Set socket address */
	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sun_family = AF_UNIX;
	unlink(tzplatform_mkpath(TZ_SYS_RUN, DCM_IPC_PATH[port]));
	DCM_SAFE_STRLCPY(serv_addr.sun_path, tzplatform_mkpath(TZ_SYS_RUN, DCM_IPC_PATH[port]), sizeof(serv_addr.sun_path));

	/* Bind socket to local address */
	for (i = 0; i < DCM_MAX_RETRY; i++) {
		if (bind(sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) == 0)
			break;

		dcm_debug("#%d bind", i);
		usleep(250000);
	}

	if (i == DCM_MAX_RETRY) {
		dcm_stderror("bind failed");
		close(sock);
		return MS_MEDIA_ERR_IPC;
	}

	dcm_debug("bind success");

	/* Listen */
	if (listen(sock, SOMAXCONN) < 0) {
		dcm_stderror("listen failed");
		close(sock);
		return MS_MEDIA_ERR_IPC;
	}
	dcm_debug("Listening...");

	/* change permission of local socket file */
	if (chmod(tzplatform_mkpath(TZ_SYS_RUN, DCM_IPC_PATH[port]), 0770) < 0)
		dcm_stderror("chmod failed");

	*socket_fd = sock;

	return MS_MEDIA_ERR_NONE;
}

int dcm_ipc_send_client_message(int socket_fd, int result, const char *msg)
{
	dcmMsg send_msg;

	DCM_CHECK_VAL(msg, MS_MEDIA_ERR_INVALID_PARAMETER);

	/* Prepare send message */
	memset((void *)&send_msg, 0, sizeof(dcmMsg));
	send_msg.msg_type = DCM_MSG_REQUEST_MEDIA;
	send_msg.result = result;
	DCM_SAFE_STRLCPY(send_msg.msg, msg, DCM_IPC_MSG_MAX_SIZE);

	/* Send msg to the socket */
	if (send(socket_fd, &send_msg, sizeof(send_msg), 0) != sizeof(send_msg)) {
		dcm_stderror("send failed");
		return MS_MEDIA_ERR_IPC;
	}

	dcm_debug("Sent result [%d]", send_msg.result);

	return MS_MEDIA_ERR_NONE;
}

int dcm_ipc_send_ready(void)
{
	int socket_fd = -1;
	struct sockaddr_un serv_addr;
	dcmMsg send_msg;

	dcm_debug_fenter();

	/* Prepare send message */
	memset((void *)&send_msg, 0, sizeof(dcmMsg));
	send_msg.msg_type = DCM_MSG_SERVER_READY;
	send_msg.uid = 0;

	/* Create a new UDS socket */
	if ((socket_fd = socket(PF_FILE, SOCK_STREAM, 0)) < 0) {
		dcm_stderror("socket failed");
		return MS_MEDIA_ERR_IPC;
	}

	/* Set dcm thread socket address */
	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sun_family = AF_UNIX;
	DCM_SAFE_STRLCPY(serv_addr.sun_path, tzplatform_mkpath(TZ_SYS_RUN, DCM_IPC_PATH[DCM_IPC_PORT_MS_RECV]), sizeof(serv_addr.sun_path));

	/* Connect to the socket */
	if (connect(socket_fd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0) {
		dcm_stderror("connect error");
		close(socket_fd);
		return MS_MEDIA_ERR_IPC;
	}

	/* Send msg to the socket */
	if (send(socket_fd, &send_msg, sizeof(send_msg), 0) != sizeof(send_msg)) {
		dcm_stderror("send failed");
		close(socket_fd);
		return MS_MEDIA_ERR_IPC;
	}

	close(socket_fd);

	dcm_debug_fleave();

	return MS_MEDIA_ERR_NONE;
}

void dcm_ipc_close_socket(int socket_fd)
{
	if (socket_fd != -1)
		close(socket_fd);
}
