/*
 * Copyright (c) 2000 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <dcm_svc_ipc.h>
#include <dcm_svc_internal.h>
#include <dcm_svc_debug.h>
#include <dcm_svc_detect_face.h>

static GMainLoop *g_dcm_svc_mainloop;

static void __dcm_svc_kill_service(void)
{
	if (g_dcm_svc_mainloop) {
		dcm_warn("Shutting down DCM Service");
		g_main_loop_quit(g_dcm_svc_mainloop);
	}
}

static gboolean __dcm_service_recieve_request(GIOChannel *src, GIOCondition condition, gpointer data)
{
	dcmMsg recv_msg;
	int sock = -1;
	int client_sock = -1;
	int ret = 0;
	int face_count = 0;

	memset((void *)&recv_msg, 0, sizeof(recv_msg));

	sock = g_io_channel_unix_get_fd(src);
	dcm_retvm_if(sock < 0, G_SOURCE_CONTINUE, "sock fd is invalid!");

	/* Accept tcp client socket */
	ret = dcm_ipc_accept_socket(sock, &client_sock);
	dcm_retvm_if(ret != MS_MEDIA_ERR_NONE, G_SOURCE_CONTINUE, "Failed to accept tcp socket! err: %d", ret);

	if (dcm_ipc_receive_message(client_sock, &recv_msg) != MS_MEDIA_ERR_NONE) {
		dcm_error("getRecvMsg failed");
		goto ERROR;
	}

	dcm_sec_info("msg_type[%d] uid[%d] msg[%.*s]", recv_msg.msg_type, recv_msg.uid, MAX_FILEPATH_LEN, recv_msg.msg);

	if (recv_msg.msg_type == DCM_MSG_REQUEST_KILL_SERVER) {
		dcm_warn("Receive DCM_MSG_REQUEST_KILL_SERVER");
		__dcm_svc_kill_service();

	} else if (recv_msg.msg_type == DCM_MSG_REQUEST_MEDIA) {
		ret = dcm_scan_single(recv_msg.msg, recv_msg.uid, &face_count);
		dcm_debug("Scan single result: ret[%d] face_count[%d]", ret, face_count);
		if (ret == MS_MEDIA_ERR_NONE)
			ret = dcm_ipc_send_client_message(client_sock, face_count, recv_msg.msg);
		else
			ret = dcm_ipc_send_client_message(client_sock, ret, recv_msg.msg);

		if (ret != MS_MEDIA_ERR_NONE)
			dcm_error("Failed to dcm_ipc_send_client_message(%d)", ret);

	} else {
		dcm_debug("Invalid message(%d)", recv_msg.msg_type);
	}

ERROR:
	dcm_ipc_close_socket(client_sock);

	return G_SOURCE_CONTINUE;
}

int main(int argc, char *argv[])
{
	int sockfd = -1;
	GSource *source = NULL;
	GIOChannel *channel = NULL;
	GMainContext *context = NULL;

	/* Create and bind new socket to main_loop */
	if (dcm_ipc_create_socket(&sockfd, DCM_IPC_PORT_DCM_RECV) != MS_MEDIA_ERR_NONE) {
		dcm_error("Failed to create socket");
		return -1;
	}

	g_dcm_svc_mainloop = g_main_loop_new(context, FALSE);
	context = g_main_loop_get_context(g_dcm_svc_mainloop);

	/* Create new channel to watch new socket for main_loop */
	channel = g_io_channel_unix_new(sockfd);
	source = g_io_create_watch(channel, G_IO_IN);

	/* Set callback to be called when socket is readable */
	g_source_set_callback(source, (GSourceFunc)__dcm_service_recieve_request, NULL, NULL);
	g_source_attach(source, context);

	if (dcm_face_detect_initialize() != MS_MEDIA_ERR_NONE)
		dcm_error("Failed to dcm_face_detect_initialize");

	/* Send ready response to dcm launcher */
	if (dcm_ipc_send_ready() != MS_MEDIA_ERR_NONE)
		dcm_error("Failed to send ready message");

	dcm_info("DCM Service is running");

	g_main_loop_run(g_dcm_svc_mainloop);

	dcm_info("DCM Service is shutting down");

	g_io_channel_shutdown(channel, FALSE, NULL);
	g_io_channel_unref(channel);
	dcm_ipc_close_socket(sockfd);
	g_main_loop_unref(g_dcm_svc_mainloop);

	dcm_face_detect_finalize();

	return 0;
}
